@@ -21,15 +21,21 @@
 #include <vte/vte.h>
 
 #include "buildsystem/ide-build-pipeline.h"
+#include "devices/ide-device-info.h"
+#include "runtimes/ide-runtime.h"
 
 G_BEGIN_DECLS
 
-void _ide_build_pipeline_cancel       (IdeBuildPipeline *self);
-void _ide_build_pipeline_set_message  (IdeBuildPipeline *self,
-                                       const gchar      *message);
-void _ide_build_pipeline_mark_broken  (IdeBuildPipeline *self);
-void _ide_build_pipeline_set_pty_size (IdeBuildPipeline *self,
-                                       guint             rows,
-                                       guint             columns);
+void _ide_build_pipeline_cancel          (IdeBuildPipeline *self);
+void _ide_build_pipeline_set_runtime     (IdeBuildPipeline *self,
+                                          IdeRuntime       *runtime);
+void _ide_build_pipeline_set_message     (IdeBuildPipeline *self,
+                                          const gchar      *message);
+void _ide_build_pipeline_mark_broken     (IdeBuildPipeline *self);
+void _ide_build_pipeline_set_device_info (IdeBuildPipeline *pipeline,
+                                          IdeDeviceInfo    *info);
+void _ide_build_pipeline_set_pty_size    (IdeBuildPipeline *self,
+                                          guint             rows,
+                                          guint             columns);
 
 G_END_DECLS
