@@ -1,6 +1,6 @@
 /* ide-thread-pool.c
  *
- * Copyright © 2015 Christian Hergert <christian@hergert.me>
+ * Copyright 2015 Christian Hergert <christian@hergert.me>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,18 +18,18 @@
 
 #define G_LOG_DOMAIN "ide-thread-pool"
 
+#include "config.h"
+
 #include <dazzle.h>
 
 #include "ide-debug.h"
 
 #include "threading/ide-thread-pool.h"
 
-#define COMPILER_MAX_THREADS (g_get_num_processors())
-#define INDEXER_MAX_THREADS  (MAX (1, g_get_num_processors() / 2))
-
 typedef struct
 {
   int type;
+  int priority;
   union {
     struct {
       GTask           *task;
@@ -42,10 +42,25 @@ typedef struct
   };
 } WorkItem;
 
+struct _IdeThreadPool
+{
+  GThreadPool       *pool;
+  IdeThreadPoolKind  kind;
+  guint              max_threads;
+  guint              worker_max_threads;
+  gboolean           exclusive;
+};
+
 DZL_DEFINE_COUNTER (TotalTasks, "ThreadPool", "Total Tasks", "Total number of tasks processed.")
 DZL_DEFINE_COUNTER (QueuedTasks, "ThreadPool", "Queued Tasks", "Current number of pending tasks.")
 
-static GThreadPool *thread_pools [IDE_THREAD_POOL_LAST];
+static IdeThreadPool thread_pools[] = {
+  { NULL, IDE_THREAD_POOL_DEFAULT, 10, 1, FALSE },
+  { NULL, IDE_THREAD_POOL_COMPILER, 1, 1, FALSE },
+  { NULL, IDE_THREAD_POOL_INDEXER,  1, 1, FALSE },
+  { NULL, IDE_THREAD_POOL_IO,       8, 1, FALSE },
+  { NULL, IDE_THREAD_POOL_LAST,     0, 0, FALSE }
+};
 
 enum {
   TYPE_TASK,
@@ -55,7 +70,7 @@ enum {
 static inline GThreadPool *
 ide_thread_pool_get_pool (IdeThreadPoolKind kind)
 {
-  return thread_pools [kind];
+  return thread_pools [kind].pool;
 }
 
 /**
@@ -91,6 +106,7 @@ ide_thread_pool_push_task (IdeThreadPoolKind  kind,
 
       work_item = g_slice_new0 (WorkItem);
       work_item->type = TYPE_TASK;
+      work_item->priority = g_task_get_priority (task);
       work_item->task.task = g_object_ref (task);
       work_item->task.func = func;
 
@@ -118,6 +134,24 @@ void
 ide_thread_pool_push (IdeThreadPoolKind kind,
                       IdeThreadFunc     func,
                       gpointer          func_data)
+{
+  ide_thread_pool_push_with_priority (kind, G_PRIORITY_DEFAULT, func, func_data);
+}
+
+/**
+ * ide_thread_pool_push_with_priority:
+ * @kind: the threadpool kind to use.
+ * @priority: the priority for func
+ * @func: (scope async) (closure func_data): A function to call in the worker thread.
+ * @func_data: user data for @func.
+ *
+ * Runs the callback on the thread pool thread.
+ */
+void
+ide_thread_pool_push_with_priority (IdeThreadPoolKind kind,
+                                    gint              priority,
+                                    IdeThreadFunc     func,
+                                    gpointer          func_data)
 {
   GThreadPool *pool;
 
@@ -137,6 +171,7 @@ ide_thread_pool_push (IdeThreadPoolKind kind,
 
       work_item = g_slice_new0 (WorkItem);
       work_item->type = TYPE_FUNC;
+      work_item->priority = priority;
       work_item->func.callback = func;
       work_item->func.data = func_data;
 
@@ -181,38 +216,43 @@ ide_thread_pool_worker (gpointer data,
   g_slice_free (WorkItem, work_item);
 }
 
+static gint
+thread_pool_sort_func (gconstpointer a,
+                       gconstpointer b,
+                       gpointer      user_data)
+{
+  const WorkItem *a_item = a;
+  const WorkItem *b_item = b;
+
+  return a_item->priority - b_item->priority;
+}
+
 void
 _ide_thread_pool_init (gboolean is_worker)
 {
-  gint compiler = COMPILER_MAX_THREADS;
-  gint indexer = INDEXER_MAX_THREADS;
-  gboolean exclusive = FALSE;
+  static gsize initialized;
 
-  if (is_worker)
+  if (g_once_init_enter (&initialized))
     {
-      compiler = 1;
-      indexer = 1;
-      exclusive = TRUE;
+      for (IdeThreadPoolKind kind = IDE_THREAD_POOL_DEFAULT;
+           kind < IDE_THREAD_POOL_LAST;
+           kind++)
+        {
+          IdeThreadPool *p = &thread_pools[kind];
+          g_autoptr(GError) error = NULL;
+
+          p->pool = g_thread_pool_new (ide_thread_pool_worker,
+                                       NULL,
+                                       is_worker ? p->worker_max_threads : p->max_threads,
+                                       p->exclusive,
+                                       &error);
+          g_thread_pool_set_sort_function (p->pool, thread_pool_sort_func, NULL);
+
+          if (error != NULL)
+            g_error ("Failed to initialize thread pool %u: %s",
+                     p->kind, error->message);
+        }
+
+      g_once_init_leave (&initialized, TRUE);
     }
-
-  /*
-   * Create our thread pool exclusive to compiler tasks (such as those from Clang).
-   * We don't want to consume threads from other GTask's such as those regarding IO so we manage
-   * these work items exclusively.
-   */
-  thread_pools [IDE_THREAD_POOL_COMPILER] = g_thread_pool_new (ide_thread_pool_worker,
-                                                               NULL,
-                                                               compiler,
-                                                               exclusive,
-                                                               NULL);
-
-  /*
-   * Create our pool exclusive to things like indexing. Such examples including building of
-   * ctags indexes or highlight indexes.
-   */
-  thread_pools [IDE_THREAD_POOL_INDEXER] = g_thread_pool_new (ide_thread_pool_worker,
-                                                              NULL,
-                                                              indexer,
-                                                              exclusive,
-                                                              NULL);
 }
