@@ -1,6 +1,6 @@
 /* ide-clang-symbol-resolver.c
  *
- * Copyright © 2015 Christian Hergert <christian@hergert.me>
+ * Copyright 2015 Christian Hergert <christian@hergert.me>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,21 +39,21 @@ ide_clang_symbol_resolver_lookup_symbol_cb (GObject      *object,
 {
   IdeClangService *service = (IdeClangService *)object;
   g_autoptr(IdeClangTranslationUnit) unit = NULL;
-  g_autoptr(GTask) task = user_data;
+  g_autoptr(IdeTask) task = user_data;
   g_autoptr(IdeSymbol) symbol = NULL;
   g_autoptr(GError) error = NULL;
   IdeSourceLocation *location;
 
   g_assert (IDE_IS_CLANG_SERVICE (service));
-  g_assert (G_IS_TASK (task));
+  g_assert (IDE_IS_TASK (task));
 
-  location = g_task_get_task_data (task);
+  location = ide_task_get_task_data (task);
 
   unit = ide_clang_service_get_translation_unit_finish (service, result, &error);
 
   if (unit == NULL)
     {
-      g_task_return_error (task, g_steal_pointer (&error));
+      ide_task_return_error (task, g_steal_pointer (&error));
       return;
     }
 
@@ -61,11 +61,11 @@ ide_clang_symbol_resolver_lookup_symbol_cb (GObject      *object,
 
   if (symbol == NULL)
     {
-      g_task_return_error (task, g_steal_pointer (&error));
+      ide_task_return_error (task, g_steal_pointer (&error));
       return;
     }
 
-  g_task_return_pointer (task, g_steal_pointer (&symbol), (GDestroyNotify)ide_symbol_unref);
+  ide_task_return_pointer (task, g_steal_pointer (&symbol), (GDestroyNotify)ide_symbol_unref);
 }
 
 static void
@@ -79,7 +79,7 @@ ide_clang_symbol_resolver_lookup_symbol_async (IdeSymbolResolver   *resolver,
   IdeClangService *service = NULL;
   IdeContext *context;
   IdeFile *file;
-  g_autoptr(GTask) task = NULL;
+  g_autoptr(IdeTask) task = NULL;
 
   IDE_ENTRY;
 
@@ -90,10 +90,10 @@ ide_clang_symbol_resolver_lookup_symbol_async (IdeSymbolResolver   *resolver,
   service = ide_context_get_service_typed (context, IDE_TYPE_CLANG_SERVICE);
   file = ide_source_location_get_file (location);
 
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_priority (task, G_PRIORITY_LOW);
-  g_task_set_task_data (task, ide_source_location_ref (location),
-                        (GDestroyNotify)ide_source_location_unref);
+  task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_priority (task, G_PRIORITY_LOW);
+  ide_task_set_task_data (task, ide_source_location_ref (location),
+                          (GDestroyNotify)ide_source_location_unref);
 
   ide_clang_service_get_translation_unit_async (service,
                                                 file,
@@ -111,14 +111,14 @@ ide_clang_symbol_resolver_lookup_symbol_finish (IdeSymbolResolver  *resolver,
                                                 GError            **error)
 {
   IdeSymbol *ret;
-  GTask *task = (GTask *)result;
+  IdeTask *task = (IdeTask *)result;
 
   IDE_ENTRY;
 
   g_return_val_if_fail (IDE_IS_CLANG_SYMBOL_RESOLVER (resolver), NULL);
-  g_return_val_if_fail (G_IS_TASK (task), NULL);
+  g_return_val_if_fail (IDE_IS_TASK (task), NULL);
 
-  ret = g_task_propagate_pointer (task, error);
+  ret = ide_task_propagate_pointer (task, error);
 
   IDE_RETURN (ret);
 }
@@ -130,15 +130,15 @@ ide_clang_symbol_resolver_get_symbol_tree_cb2 (GObject      *object,
 {
   IdeClangTranslationUnit *unit = (IdeClangTranslationUnit *)object;
   g_autoptr(IdeSymbolTree) ret = NULL;
-  g_autoptr(GTask) task = user_data;
+  g_autoptr(IdeTask) task = user_data;
   g_autoptr(GError) error = NULL;
 
   ret = ide_clang_translation_unit_get_symbol_tree_finish (unit, result, &error);
 
   if (ret == NULL)
-    g_task_return_error (task, g_steal_pointer (&error));
+    ide_task_return_error (task, g_steal_pointer (&error));
   else
-    g_task_return_pointer (task, g_steal_pointer (&ret), g_object_unref);
+    ide_task_return_pointer (task, g_steal_pointer (&ret), g_object_unref);
 }
 
 static void
@@ -148,29 +148,29 @@ ide_clang_symbol_resolver_get_symbol_tree_cb (GObject      *object,
 {
   IdeClangService *service = (IdeClangService *)object;
   g_autoptr(IdeClangTranslationUnit) unit = NULL;
-  g_autoptr(GTask) task = user_data;
+  g_autoptr(IdeTask) task = user_data;
   g_autoptr(GError) error = NULL;
   GFile *file;
 
   IDE_ENTRY;
 
   g_assert (IDE_IS_CLANG_SERVICE (service));
-  g_assert (G_IS_TASK (task));
+  g_assert (IDE_IS_TASK (task));
 
   unit = ide_clang_service_get_translation_unit_finish (service, result, &error);
 
   if (unit == NULL)
     {
-      g_task_return_error (task, g_steal_pointer (&error));
+      ide_task_return_error (task, g_steal_pointer (&error));
       IDE_EXIT;
     }
 
-  file = g_task_get_task_data (task);
+  file = ide_task_get_task_data (task);
   g_assert (G_IS_FILE (file));
 
   ide_clang_translation_unit_get_symbol_tree_async (unit,
                                                     file,
-                                                    g_task_get_cancellable (task),
+                                                    ide_task_get_cancellable (task),
                                                     ide_clang_symbol_resolver_get_symbol_tree_cb2,
                                                     g_object_ref (task));
 
@@ -186,7 +186,7 @@ ide_clang_symbol_resolver_get_symbol_tree_async (IdeSymbolResolver   *resolver,
                                                  gpointer             user_data)
 {
   IdeClangSymbolResolver *self = (IdeClangSymbolResolver *)resolver;
-  g_autoptr(GTask) task = NULL;
+  g_autoptr(IdeTask) task = NULL;
   g_autoptr(IdeFile) ifile = NULL;
   IdeClangService *service;
   IdeContext *context;
@@ -200,9 +200,9 @@ ide_clang_symbol_resolver_get_symbol_tree_async (IdeSymbolResolver   *resolver,
   context = ide_object_get_context (IDE_OBJECT (self));
   service = ide_context_get_service_typed (context, IDE_TYPE_CLANG_SERVICE);
 
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_priority (task, G_PRIORITY_LOW);
-  g_task_set_task_data (task, g_object_ref (file), g_object_unref);
+  task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_priority (task, G_PRIORITY_LOW);
+  ide_task_set_task_data (task, g_object_ref (file), g_object_unref);
 
   ifile = ide_file_new (context, file);
 
@@ -222,14 +222,14 @@ ide_clang_symbol_resolver_get_symbol_tree_finish (IdeSymbolResolver  *resolver,
                                                   GError            **error)
 {
   IdeSymbolTree *ret;
-  GTask *task = (GTask *)result;
+  IdeTask *task = (IdeTask *)result;
 
   IDE_ENTRY;
 
   g_return_val_if_fail (IDE_IS_CLANG_SYMBOL_RESOLVER (resolver), NULL);
-  g_return_val_if_fail (G_IS_TASK (task), NULL);
+  g_return_val_if_fail (IDE_IS_TASK (task), NULL);
 
-  ret = g_task_propagate_pointer (task, error);
+  ret = ide_task_propagate_pointer (task, error);
 
   IDE_RETURN (ret);
 }
@@ -241,32 +241,32 @@ ide_clang_symbol_resolver_find_scope_cb (GObject      *object,
 {
   IdeClangService *service = (IdeClangService *)object;
   g_autoptr(IdeClangTranslationUnit) unit = NULL;
-  g_autoptr(GTask) task = user_data;
+  g_autoptr(IdeTask) task = user_data;
   g_autoptr(IdeSymbol) symbol = NULL;
   g_autoptr(GError) error = NULL;
   IdeSourceLocation *location;
 
   g_assert (IDE_IS_CLANG_SERVICE (service));
   g_assert (G_IS_ASYNC_RESULT (result));
-  g_assert (G_IS_TASK (task));
+  g_assert (IDE_IS_TASK (task));
 
   unit = ide_clang_service_get_translation_unit_finish (service, result, &error);
 
   if (unit == NULL)
     {
-      g_task_return_error (task, g_steal_pointer (&error));
+      ide_task_return_error (task, g_steal_pointer (&error));
       return;
     }
 
-  location = g_task_get_task_data (task);
+  location = ide_task_get_task_data (task);
   symbol = ide_clang_translation_unit_find_nearest_scope (unit, location, &error);
 
   if (symbol == NULL)
-    g_task_return_error (task, g_steal_pointer (&error));
+    ide_task_return_error (task, g_steal_pointer (&error));
   else
-    g_task_return_pointer (task,
-                           g_steal_pointer (&symbol),
-                           (GDestroyNotify) ide_symbol_unref);
+    ide_task_return_pointer (task,
+                             g_steal_pointer (&symbol),
+                             (GDestroyNotify) ide_symbol_unref);
 }
 
 static void
@@ -277,7 +277,7 @@ ide_clang_symbol_resolver_find_nearest_scope_async (IdeSymbolResolver   *symbol_
                                                     gpointer             user_data)
 {
   IdeClangSymbolResolver *self = (IdeClangSymbolResolver *)symbol_resolver;
-  g_autoptr(GTask) task = NULL;
+  g_autoptr(IdeTask) task = NULL;
   IdeClangService *service;
   IdeContext *context;
   IdeFile *file;
@@ -287,12 +287,12 @@ ide_clang_symbol_resolver_find_nearest_scope_async (IdeSymbolResolver   *symbol_
   g_return_if_fail (IDE_IS_CLANG_SYMBOL_RESOLVER (self));
   g_return_if_fail (location != NULL);
 
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_priority (task, G_PRIORITY_LOW);
-  g_task_set_source_tag (task, ide_clang_symbol_resolver_find_nearest_scope_async);
-  g_task_set_task_data (task,
-                        ide_source_location_ref (location),
-                        (GDestroyNotify) ide_source_location_unref);
+  task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_priority (task, G_PRIORITY_LOW);
+  ide_task_set_source_tag (task, ide_clang_symbol_resolver_find_nearest_scope_async);
+  ide_task_set_task_data (task,
+                          ide_source_location_ref (location),
+                          (GDestroyNotify) ide_source_location_unref);
 
   context = ide_object_get_context (IDE_OBJECT (self));
   service = ide_context_get_service_typed (context, IDE_TYPE_CLANG_SERVICE);
@@ -318,9 +318,9 @@ ide_clang_symbol_resolver_find_nearest_scope_finish (IdeSymbolResolver  *resolve
   IDE_ENTRY;
 
   g_return_val_if_fail (IDE_IS_CLANG_SYMBOL_RESOLVER (resolver), NULL);
-  g_return_val_if_fail (G_IS_TASK (result), NULL);
+  g_return_val_if_fail (IDE_IS_TASK (result), NULL);
 
-  ret = g_task_propagate_pointer (G_TASK (result), error);
+  ret = ide_task_propagate_pointer (IDE_TASK (result), error);
 
   IDE_RETURN (ret);
 }
