@@ -1,6 +1,6 @@
 /* gbp-flatpak-runtime-provider.c
  *
- * Copyright © 2016 Christian Hergert <chergert@redhat.com>
+ * Copyright 2016 Christian Hergert <chergert@redhat.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -207,9 +207,10 @@ gbp_flatpak_runtime_provider_unload (IdeRuntimeProvider *provider,
   g_assert (GBP_IS_FLATPAK_RUNTIME_PROVIDER (self));
   g_assert (IDE_IS_RUNTIME_MANAGER (manager));
 
-  g_signal_handlers_disconnect_by_func (app_addin,
-                                        G_CALLBACK (runtime_added_cb),
-                                        self);
+  if (app_addin != NULL)
+    g_signal_handlers_disconnect_by_func (app_addin,
+                                          G_CALLBACK (runtime_added_cb),
+                                          self);
 
   if (self->runtimes != NULL)
     {
@@ -254,7 +255,7 @@ gbp_flatpak_runtime_provider_install_cb (GObject      *object,
                                          gpointer      user_data)
 {
   IdeTransferManager *transfer_manager = (IdeTransferManager *)object;
-  g_autoptr(GTask) task = user_data;
+  g_autoptr(IdeTask) task = user_data;
   g_autoptr(GError) error = NULL;
   InstallRuntime *install;
 
@@ -262,15 +263,15 @@ gbp_flatpak_runtime_provider_install_cb (GObject      *object,
 
   g_assert (IDE_IS_TRANSFER_MANAGER (transfer_manager));
   g_assert (G_IS_ASYNC_RESULT (result));
-  g_assert (G_IS_TASK (task));
+  g_assert (IDE_IS_TASK (task));
 
-  install = g_task_get_task_data (task);
+  install = ide_task_get_task_data (task);
 
   if (!ide_transfer_manager_execute_finish (transfer_manager, result, &error))
     {
       if (!install->failed)
         {
-          g_task_return_error (task, g_steal_pointer (&error));
+          ide_task_return_error (task, g_steal_pointer (&error));
           install->failed = TRUE;
         }
     }
@@ -278,7 +279,7 @@ gbp_flatpak_runtime_provider_install_cb (GObject      *object,
   install->op_count--;
 
   if (install->op_count == 0 && !install->failed)
-    g_task_return_boolean (task, TRUE);
+    ide_task_return_boolean (task, TRUE);
 
   IDE_EXIT;
 }
@@ -289,7 +290,7 @@ gbp_flatpak_runtime_provider_install_docs_cb (GObject      *object,
                                               gpointer      user_data)
 {
   IdeTransferManager *transfer_manager = (IdeTransferManager *)object;
-  g_autoptr(GTask) task = user_data;
+  g_autoptr(IdeTask) task = user_data;
   g_autoptr(GError) error = NULL;
   InstallRuntime *install;
 
@@ -297,9 +298,9 @@ gbp_flatpak_runtime_provider_install_docs_cb (GObject      *object,
 
   g_assert (IDE_IS_TRANSFER_MANAGER (transfer_manager));
   g_assert (G_IS_ASYNC_RESULT (result));
-  g_assert (G_IS_TASK (task));
+  g_assert (IDE_IS_TASK (task));
 
-  install = g_task_get_task_data (task);
+  install = ide_task_get_task_data (task);
 
   /* This error is not fatal */
   if (!ide_transfer_manager_execute_finish (transfer_manager, result, &error))
@@ -308,7 +309,7 @@ gbp_flatpak_runtime_provider_install_docs_cb (GObject      *object,
   install->op_count--;
 
   if (install->op_count == 0 && !install->failed)
-    g_task_return_boolean (task, TRUE);
+    ide_task_return_boolean (task, TRUE);
 
   IDE_EXIT;
 }
@@ -319,7 +320,7 @@ gbp_flatpak_runtime_provider_locate_sdk_cb (GObject      *object,
                                             gpointer      user_data)
 {
   GbpFlatpakApplicationAddin *app_addin = (GbpFlatpakApplicationAddin *)object;
-  g_autoptr(GTask) task = user_data;
+  g_autoptr(IdeTask) task = user_data;
   g_autoptr(GError) error = NULL;
   g_autofree gchar *docs_id = NULL;
   IdeTransferManager *transfer_manager;
@@ -331,11 +332,11 @@ gbp_flatpak_runtime_provider_locate_sdk_cb (GObject      *object,
 
   g_assert (GBP_IS_FLATPAK_APPLICATION_ADDIN (app_addin));
   g_assert (G_IS_ASYNC_RESULT (result));
-  g_assert (G_IS_TASK (task));
-  g_assert (!g_task_get_completed (task));
+  g_assert (IDE_IS_TASK (task));
+  g_assert (!ide_task_get_completed (task));
 
-  install = g_task_get_task_data (task);
-  cancellable = g_task_get_cancellable (task);
+  install = ide_task_get_task_data (task);
+  cancellable = ide_task_get_cancellable (task);
 
   g_assert (install != NULL);
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
@@ -351,7 +352,7 @@ gbp_flatpak_runtime_provider_locate_sdk_cb (GObject      *object,
     {
       if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
         {
-          g_task_return_error (task, g_steal_pointer (&error));
+          ide_task_return_error (task, g_steal_pointer (&error));
           IDE_EXIT;
         }
     }
@@ -429,7 +430,7 @@ gbp_flatpak_runtime_provider_locate_sdk_cb (GObject      *object,
 
   /* Complete the task now if everything is done */
   if (install->op_count == 0)
-    g_task_return_boolean (task, TRUE);
+    ide_task_return_boolean (task, TRUE);
 
   IDE_EXIT;
 }
@@ -442,7 +443,7 @@ gbp_flatpak_runtime_provider_install_async (IdeRuntimeProvider  *provider,
                                             gpointer             user_data)
 {
   GbpFlatpakRuntimeProvider *self = (GbpFlatpakRuntimeProvider *)provider;
-  g_autoptr(GTask) task = NULL;
+  g_autoptr(IdeTask) task = NULL;
   g_autofree gchar *id = NULL;
   g_autofree gchar *arch = NULL;
   g_autofree gchar *branch = NULL;
@@ -462,9 +463,9 @@ gbp_flatpak_runtime_provider_install_async (IdeRuntimeProvider  *provider,
    * done before completing the operation.
    */
 
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_source_tag (task, gbp_flatpak_runtime_provider_install_async);
-  g_task_set_priority (task, G_PRIORITY_LOW);
+  task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_source_tag (task, gbp_flatpak_runtime_provider_install_async);
+  ide_task_set_priority (task, G_PRIORITY_LOW);
 
   if (!g_str_has_prefix (runtime_id, "flatpak:"))
     IDE_GOTO (unknown_runtime_id);
@@ -477,7 +478,7 @@ gbp_flatpak_runtime_provider_install_async (IdeRuntimeProvider  *provider,
   install->arch = g_steal_pointer (&arch);
   install->branch = g_steal_pointer (&branch);
 
-  g_task_set_task_data (task, install, (GDestroyNotify)install_runtime_free);
+  ide_task_set_task_data (task, install, (GDestroyNotify)install_runtime_free);
 
   gbp_flatpak_application_addin_locate_sdk_async (gbp_flatpak_application_addin_get_default (),
                                                   install->id,
@@ -490,11 +491,11 @@ gbp_flatpak_runtime_provider_install_async (IdeRuntimeProvider  *provider,
   IDE_EXIT;
 
 unknown_runtime_id:
-  g_task_return_new_error (task,
-                           G_IO_ERROR,
-                           G_IO_ERROR_NOT_SUPPORTED,
-                           "Unknown runtime_id %s",
-                           runtime_id);
+  ide_task_return_new_error (task,
+                             G_IO_ERROR,
+                             G_IO_ERROR_NOT_SUPPORTED,
+                             "Unknown runtime_id %s",
+                             runtime_id);
 
   IDE_EXIT;
 }
@@ -509,9 +510,9 @@ gbp_flatpak_runtime_provider_install_finish (IdeRuntimeProvider  *provider,
   IDE_ENTRY;
 
   g_assert (GBP_IS_FLATPAK_RUNTIME_PROVIDER (provider));
-  g_assert (G_IS_TASK (result));
+  g_assert (IDE_IS_TASK (result));
 
-  ret = g_task_propagate_boolean (G_TASK (result), error);
+  ret = ide_task_propagate_boolean (IDE_TASK (result), error);
 
   IDE_RETURN (ret);
 }
@@ -522,18 +523,18 @@ gbp_flatpak_runtime_provider_bootstrap_cb (GObject      *object,
                                            gpointer      user_data)
 {
   GbpFlatpakRuntimeProvider *self;
-  g_autoptr(GTask) task = user_data;
+  g_autoptr(IdeTask) task = user_data;
   g_autoptr(GError) error = NULL;
   BootstrapState *state;
 
   g_assert (IDE_IS_MAIN_THREAD ());
   g_assert (GBP_IS_FLATPAK_RUNTIME_PROVIDER (object) ||
             IDE_IS_TRANSFER_MANAGER (object));
-  g_assert (G_IS_TASK (task));
+  g_assert (IDE_IS_TASK (task));
   g_assert (G_IS_ASYNC_RESULT (result));
 
-  self = g_task_get_source_object (task);
-  state = g_task_get_task_data (task);
+  self = ide_task_get_source_object (task);
+  state = ide_task_get_task_data (task);
 
   g_assert (GBP_IS_FLATPAK_RUNTIME_PROVIDER (self));
   g_assert (state != NULL);
@@ -546,8 +547,8 @@ gbp_flatpak_runtime_provider_bootstrap_cb (GObject      *object,
       if (!gbp_flatpak_runtime_provider_install_finish (IDE_RUNTIME_PROVIDER (object), result, &error))
         {
           g_warning ("Failed to install runtime: %s", error->message);
-          if (!g_task_get_completed (task))
-            g_task_return_error (task, g_steal_pointer (&error));
+          if (!ide_task_get_completed (task))
+            ide_task_return_error (task, g_steal_pointer (&error));
           return;
         }
     }
@@ -556,16 +557,16 @@ gbp_flatpak_runtime_provider_bootstrap_cb (GObject      *object,
       if (!ide_transfer_manager_execute_finish (IDE_TRANSFER_MANAGER (object), result, &error))
         {
           g_warning ("Failed to install runtime: %s", error->message);
-          if (!g_task_get_completed (task))
-            g_task_return_error (task, g_steal_pointer (&error));
+          if (!ide_task_get_completed (task))
+            ide_task_return_error (task, g_steal_pointer (&error));
           return;
         }
     }
 
-  if (g_task_return_error_if_cancelled (task))
+  if (ide_task_return_error_if_cancelled (task))
     return;
 
-  if (state->count == 0 && !g_task_get_completed (task))
+  if (state->count == 0 && !ide_task_get_completed (task))
     {
       g_autofree gchar *runtime_id = NULL;
       IdeRuntimeManager *runtime_manager;
@@ -582,12 +583,12 @@ gbp_flatpak_runtime_provider_bootstrap_cb (GObject      *object,
       runtime = ide_runtime_manager_get_runtime (runtime_manager, runtime_id);
 
       if (runtime == NULL)
-        g_task_return_new_error (task,
-                                 G_IO_ERROR,
-                                 G_IO_ERROR_NOT_SUPPORTED,
-                                 "Falling back to default runtime lookup");
+        ide_task_return_new_error (task,
+                                   G_IO_ERROR,
+                                   G_IO_ERROR_NOT_SUPPORTED,
+                                   "Falling back to default runtime lookup");
       else
-        g_task_return_pointer (task, g_object_ref (runtime), g_object_unref);
+        ide_task_return_pointer (task, g_object_ref (runtime), g_object_unref);
     }
 }
 
@@ -602,7 +603,7 @@ gbp_flatpak_runtime_provider_bootstrap_async (IdeRuntimeProvider  *provider,
   g_autofree gchar *name = NULL;
   g_autofree gchar *arch = NULL;
   g_autofree gchar *branch = NULL;
-  g_autoptr(GTask) task = NULL;
+  g_autoptr(IdeTask) task = NULL;
   IdeConfiguration *config;
   BootstrapState *state;
   const gchar *runtime_id;
@@ -614,9 +615,9 @@ gbp_flatpak_runtime_provider_bootstrap_async (IdeRuntimeProvider  *provider,
   g_assert (IDE_IS_BUILD_PIPELINE (pipeline));
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_source_tag (task, gbp_flatpak_runtime_provider_bootstrap_async);
-  g_task_set_priority (task, G_PRIORITY_LOW);
+  task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_source_tag (task, gbp_flatpak_runtime_provider_bootstrap_async);
+  ide_task_set_priority (task, G_PRIORITY_LOW);
 
   build_arch = ide_build_pipeline_get_arch (pipeline);
   config = ide_build_pipeline_get_configuration (pipeline);
@@ -626,10 +627,10 @@ gbp_flatpak_runtime_provider_bootstrap_async (IdeRuntimeProvider  *provider,
       !g_str_has_prefix (runtime_id, "flatpak:") ||
       !gbp_flatpak_split_id (runtime_id + strlen ("flatpak:"), &name, &arch, &branch))
     {
-      g_task_return_new_error (task,
-                               G_IO_ERROR,
-                               G_IO_ERROR_NOT_FOUND,
-                               "No runtime available");
+      ide_task_return_new_error (task,
+                                 G_IO_ERROR,
+                                 G_IO_ERROR_NOT_FOUND,
+                                 "No runtime available");
       IDE_EXIT;
     }
 
@@ -639,7 +640,7 @@ gbp_flatpak_runtime_provider_bootstrap_async (IdeRuntimeProvider  *provider,
   state->name = g_steal_pointer (&name);
   state->branch = g_steal_pointer (&branch);
   state->arch = g_strdup (build_arch);
-  g_task_set_task_data (task, state, bootstrap_state_free);
+  ide_task_set_task_data (task, state, bootstrap_state_free);
 
   if (GBP_IS_FLATPAK_MANIFEST (state->config))
     {
@@ -702,9 +703,9 @@ gbp_flatpak_runtime_provider_bootstrap_finish (IdeRuntimeProvider  *provider,
   IDE_ENTRY;
 
   g_assert (GBP_IS_FLATPAK_RUNTIME_PROVIDER (provider));
-  g_assert (G_IS_TASK (result));
+  g_assert (IDE_IS_TASK (result));
 
-  ret = g_task_propagate_pointer (G_TASK (result), error);
+  ret = ide_task_propagate_pointer (IDE_TASK (result), error);
 
   IDE_RETURN (ret);
 }
@@ -718,7 +719,7 @@ gbp_flatpak_runtime_provider_locate_sdk_async (GbpFlatpakRuntimeProvider *self,
                                                GAsyncReadyCallback        callback,
                                                gpointer                   user_data)
 {
-  g_autoptr(GTask) task = NULL;
+  g_autoptr(IdeTask) task = NULL;
 
   IDE_ENTRY;
 
@@ -728,9 +729,9 @@ gbp_flatpak_runtime_provider_locate_sdk_async (GbpFlatpakRuntimeProvider *self,
   g_assert (branch != NULL);
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_source_tag (task, gbp_flatpak_runtime_provider_locate_sdk_async);
-  g_task_set_priority (task, G_PRIORITY_LOW);
+  task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_source_tag (task, gbp_flatpak_runtime_provider_locate_sdk_async);
+  ide_task_set_priority (task, G_PRIORITY_LOW);
 
   IDE_EXIT;
 }
