@@ -1,6 +1,6 @@
 /* ide-xml-completion-values.c
  *
- * Copyright © 2017 Sebastien Lafargue <slafargue@gnome.org>
+ * Copyright 2017 Sebastien Lafargue <slafargue@gnome.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +23,8 @@ typedef struct _MatchingState
 {
   IdeXmlRngDefine  *define;
   GPtrArray        *match_values;
-  const gchar      *values;
-  const gchar      *prefix;
+  gchar            *values;
+  gchar            *prefix;
 
   guint             is_initial_state : 1;
 } MatchingState;
@@ -95,8 +95,8 @@ matching_state_new (IdeXmlRngDefine  *define,
   state = g_slice_new0 (MatchingState);
 
   state->define = define;
-  state->values = (values != NULL) ? g_strdup (values) : NULL;
-  state->prefix = (prefix != NULL) ? g_strdup (prefix) : NULL;
+  state->values = g_strdup (values);
+  state->prefix = g_strdup (prefix);
 
   state->is_initial_state = FALSE;
 
@@ -108,6 +108,7 @@ matching_state_free (MatchingState *state)
 {
   g_clear_pointer (&state->values, g_free);
   g_clear_pointer (&state->prefix, g_free);
+  g_slice_free (MatchingState, state);
 }
 
 static GPtrArray *
