@@ -1,6 +1,6 @@
 /* ide-xml-parser.c
  *
- * Copyright © 2017 Sebastien Lafargue <slafargue@gnome.org>
+ * Copyright 2017 Sebastien Lafargue <slafargue@gnome.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,16 +49,18 @@ color_tag_free (gpointer *data)
   g_free (tag->bg);
 }
 
+#define STATIC_COLOR_TAG(a,b,c) \
+  { (gchar*)a, (gchar *)b, (gchar *)c }
+
 /* Keep it in sync with ColorTagId enum */
-static ColorTag default_color_tags [] =
-{
-  { "label",       "#000000", "#D5E7FC" }, // COLOR_TAG_LABEL
-  { "id",          "#000000", "#D9E7BD" }, // COLOR_TAG_ID
-  { "style-class", "#000000", "#DFCD9B" }, // COLOR_TAG_STYLE_CLASS
-  { "type",        "#000000", "#F4DAC3" }, // COLOR_TAG_TYPE
-  { "parent",      "#000000", "#DEBECF" }, // COLOR_TAG_PARENT
-  { "class",       "#000000", "#FFEF98" }, // COLOR_TAG_CLASS
-  { "attribute",   "#000000", "#F0E68C" }, // COLOR_TAG_ATTRIBUTE
+static ColorTag default_color_tags [] = {
+  STATIC_COLOR_TAG ("label",       "#000000", "#D5E7FC" ), // COLOR_TAG_LABEL
+  STATIC_COLOR_TAG ("id",          "#000000", "#D9E7BD" ), // COLOR_TAG_ID
+  STATIC_COLOR_TAG ("style-class", "#000000", "#DFCD9B" ), // COLOR_TAG_STYLE_CLASS
+  STATIC_COLOR_TAG ("type",        "#000000", "#F4DAC3" ), // COLOR_TAG_TYPE
+  STATIC_COLOR_TAG ("parent",      "#000000", "#DEBECF" ), // COLOR_TAG_PARENT
+  STATIC_COLOR_TAG ("class",       "#000000", "#FFEF98" ), // COLOR_TAG_CLASS
+  STATIC_COLOR_TAG ("attribute",   "#000000", "#F0E68C" ), // COLOR_TAG_ATTRIBUTE
   { NULL },
 };
 
@@ -73,7 +75,7 @@ parser_state_free (ParserState *state)
   g_clear_object (&state->stack);
 
   g_clear_pointer (&state->content, g_bytes_unref);
-  g_clear_pointer (&state->schemas, g_array_unref);
+  g_clear_pointer (&state->schemas, g_ptr_array_unref);
 
   g_slice_free (ParserState, state);
 }
@@ -540,7 +542,7 @@ ide_xml_parser_characters_sax_cb (ParserState    *state,
 }
 
 static void
-ide_xml_parser_get_analysis_worker (GTask        *task,
+ide_xml_parser_get_analysis_worker (IdeTask      *task,
                                     gpointer      source_object,
                                     gpointer      task_data,
                                     GCancellable *cancellable)
@@ -555,11 +557,11 @@ ide_xml_parser_get_analysis_worker (GTask        *task,
   gsize doc_size;
 
   g_assert (IDE_IS_XML_PARSER (self));
-  g_assert (G_IS_TASK (task));
+  g_assert (IDE_IS_TASK (task));
   g_assert (state != NULL);
   g_assert (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
-  if (g_task_return_error_if_cancelled (task))
+  if (ide_task_return_error_if_cancelled (task))
     return;
 
   doc_data = g_bytes_get_data (state->content, &doc_size);
@@ -579,10 +581,10 @@ ide_xml_parser_get_analysis_worker (GTask        *task,
   analysis = g_steal_pointer (&state->analysis);
   if (analysis == NULL)
     {
-      g_task_return_new_error (task,
-                               G_IO_ERROR,
-                               G_IO_ERROR_FAILED,
-                               _("Failed to create the XML tree."));
+      ide_task_return_new_error (task,
+                                 G_IO_ERROR,
+                                 G_IO_ERROR_FAILED,
+                                 _("Failed to create the XML tree."));
       return;
     }
 
@@ -602,7 +604,7 @@ ide_xml_parser_get_analysis_worker (GTask        *task,
     ide_xml_analysis_set_schemas (analysis, g_steal_pointer (&state->schemas));
 
   ide_xml_analysis_set_sequence (analysis, state->sequence);
-  g_task_return_pointer (task, analysis, (GDestroyNotify)ide_xml_analysis_unref);
+  ide_task_return_pointer (task, analysis, (GDestroyNotify)ide_xml_analysis_unref);
 }
 
 void
@@ -614,14 +616,15 @@ ide_xml_parser_get_analysis_async (IdeXmlParser        *self,
                                    GAsyncReadyCallback  callback,
                                    gpointer             user_data)
 {
-  g_autoptr(GTask) task = NULL;
+  g_autoptr(IdeTask) task = NULL;
   ParserState *state;
 
   g_return_if_fail (IDE_IS_XML_PARSER (self));
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_source_tag (task, ide_xml_parser_get_analysis_async);
+  task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_source_tag (task, ide_xml_parser_get_analysis_async);
+  ide_task_set_kind (task, IDE_TASK_KIND_INDEXER);
 
   state = g_slice_new0 (ParserState);
   state->self = self;
@@ -642,8 +645,8 @@ ide_xml_parser_get_analysis_async (IdeXmlParser        *self,
   state->parent_node = state->root_node;
   ide_xml_stack_push (state->stack, "root", state->root_node, NULL, 0);
 
-  g_task_set_task_data (task, state, (GDestroyNotify)parser_state_free);
-  g_task_run_in_thread (task, ide_xml_parser_get_analysis_worker);
+  ide_task_set_task_data (task, state, (GDestroyNotify)parser_state_free);
+  ide_task_run_in_thread (task, ide_xml_parser_get_analysis_worker);
 }
 
 IdeXmlAnalysis *
@@ -652,9 +655,9 @@ ide_xml_parser_get_analysis_finish (IdeXmlParser  *self,
                                     GError       **error)
 {
   g_return_val_if_fail (IDE_IS_XML_PARSER (self), NULL);
-  g_return_val_if_fail (G_IS_TASK (result), NULL);
+  g_return_val_if_fail (IDE_IS_TASK (result), NULL);
 
-  return g_task_propagate_pointer (G_TASK (result), error);
+  return ide_task_propagate_pointer (IDE_TASK (result), error);
 }
 
 gchar *
