@@ -1,6 +1,6 @@
 /* ide-xml-service.c
  *
- * Copyright © 2017 Sébastien Lafargue <slafargue@gnome.org>
+ * Copyright 2017 Sébastien Lafargue <slafargue@gnome.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,8 +47,8 @@ struct _IdeXmlService
 
 static void service_iface_init (IdeServiceInterface *iface);
 
-G_DEFINE_DYNAMIC_TYPE_EXTENDED (IdeXmlService, ide_xml_service, IDE_TYPE_OBJECT, 0,
-                                G_IMPLEMENT_INTERFACE (IDE_TYPE_SERVICE, service_iface_init))
+G_DEFINE_TYPE_WITH_CODE (IdeXmlService, ide_xml_service, IDE_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (IDE_TYPE_SERVICE, service_iface_init))
 
 static void
 ide_xml_service_build_tree_cb2 (GObject      *object,
@@ -362,7 +362,7 @@ ide_xml_service_get_analysis_async (IdeXmlService       *self,
                               g_steal_pointer (&task));
 }
 
-IdeXmlAnalysis *
+static IdeXmlAnalysis *
 ide_xml_service_get_analysis_finish (IdeXmlService  *self,
                                      GAsyncResult   *result,
                                      GError        **error)
@@ -539,18 +539,20 @@ ide_xml_service_get_diagnostics_async (IdeXmlService       *self,
   g_autoptr(GTask) task = NULL;
   IdeXmlAnalysis *cached;
 
+  g_return_if_fail (IDE_IS_MAIN_THREAD ());
   g_return_if_fail (IDE_IS_XML_SERVICE (self));
   g_return_if_fail (IDE_IS_FILE (ifile));
   g_return_if_fail (IDE_IS_BUFFER (buffer) || buffer == NULL);
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
   task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, ide_xml_service_get_diagnostics_async);
 
   /*
    * If we have a cached analysis with some diagnostics,
    * and it is new enough, then re-use it.
    */
-  if (NULL != (cached = dzl_task_cache_peek (self->analyses, ifile)))
+  if ((cached = dzl_task_cache_peek (self->analyses, ifile)))
     {
       IdeContext *context;
       IdeUnsavedFiles *unsaved_files;
@@ -562,13 +564,14 @@ ide_xml_service_get_diagnostics_async (IdeXmlService       *self,
       context = ide_object_get_context (IDE_OBJECT (self));
       unsaved_files = ide_context_get_unsaved_files (context);
 
-      if (NULL != (uf = ide_unsaved_files_get_unsaved_file (unsaved_files, gfile)) &&
+      if ((uf = ide_unsaved_files_get_unsaved_file (unsaved_files, gfile)) &&
           ide_xml_analysis_get_sequence (cached) == ide_unsaved_file_get_sequence (uf))
         {
-          diagnostics = ide_diagnostics_ref (ide_xml_analysis_get_diagnostics (cached));
+          diagnostics = ide_xml_analysis_get_diagnostics (cached);
           g_assert (diagnostics != NULL);
-
-          g_task_return_pointer (task, diagnostics, (GDestroyNotify)ide_diagnostics_unref);
+          g_task_return_pointer (task,
+                                 ide_diagnostics_ref (diagnostics),
+                                 (GDestroyNotify)ide_diagnostics_unref);
           return;
         }
     }
@@ -594,13 +597,11 @@ ide_xml_service_get_diagnostics_finish (IdeXmlService  *self,
                                         GAsyncResult   *result,
                                         GError        **error)
 {
-  GTask *task = (GTask *)result;
-
   g_return_val_if_fail (IDE_IS_XML_SERVICE (self), NULL);
   g_return_val_if_fail (G_IS_TASK (result), NULL);
   g_return_val_if_fail (error != NULL, NULL);
 
-  return g_task_propagate_pointer (task, error);
+  return g_task_propagate_pointer (G_TASK (result), error);
 }
 
 static void
@@ -625,20 +626,21 @@ ide_xml_service_context_loaded (IdeService *service)
 
 typedef struct
 {
-  GTask         *task;
-  IdeFile       *ifile;
-  IdeBuffer     *buffer;
-  gint           line;
-  gint           line_offset;
+  IdeFile   *ifile;
+  IdeBuffer *buffer;
+  gint       line;
+  gint       line_offset;
 } PositionState;
 
 static void
 position_state_free (PositionState *state)
 {
   g_assert (state != NULL);
+  g_assert (IDE_IS_MAIN_THREAD ());
 
-  g_object_unref (state->ifile);
-  g_object_unref (state->buffer);
+  g_clear_object (&state->ifile);
+  g_clear_object (&state->buffer);
+  g_slice_free (PositionState, state);
 }
 
 static inline gboolean
@@ -806,7 +808,7 @@ get_position (IdeXmlService   *self,
               gint             line,
               gint             line_offset)
 {
-  IdeXmlPosition *position;
+  IdeXmlPosition *position = NULL;
   IdeXmlSymbolNode *root_node;
   IdeXmlSymbolNode *current_node, *child_node;
   IdeXmlSymbolNode *candidate_node = NULL;
@@ -963,27 +965,36 @@ ide_xml_service_get_position_from_cursor_cb (GObject      *object,
                                              gpointer      user_data)
 {
   IdeXmlService *self = (IdeXmlService *)object;
-  PositionState *state = (PositionState *)user_data;
-  g_autoptr(GTask) task = state->task;
-  IdeXmlPosition *position;
-  IdeXmlAnalysis *analysis = NULL;
+  g_autoptr(IdeXmlPosition) position = NULL;
+  g_autoptr(IdeTask) task = user_data;
   g_autoptr(GError) error = NULL;
+  IdeXmlAnalysis *analysis;
+  PositionState *state;
 
   IDE_ENTRY;
 
-  g_assert (G_IS_TASK (task));
+  g_assert (IDE_IS_MAIN_THREAD ());
+  g_assert (IDE_IS_TASK (task));
+  g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (IDE_IS_XML_SERVICE (self));
 
-  analysis = ide_xml_service_get_analysis_finish (self, result, &error);
-  if (analysis != NULL)
+  if (!(analysis = ide_xml_service_get_analysis_finish (self, result, &error)))
     {
-      position = get_position (self, analysis, (GtkTextBuffer *)state->buffer, state->line, state->line_offset);
-      g_task_return_pointer (task, position, g_object_unref);
+      ide_task_return_error (task, g_steal_pointer (&error));
+      return;
     }
-  else
-    g_task_return_error (task, g_steal_pointer (&error));
 
-  position_state_free (state);
+  state = ide_task_get_task_data (task);
+
+  position = get_position (self,
+                           analysis,
+                           GTK_TEXT_BUFFER (state->buffer),
+                           state->line,
+                           state->line_offset);
+
+  ide_task_return_pointer (task,
+                           g_steal_pointer (&position),
+                           g_object_unref);
 
   IDE_EXIT;
 }
@@ -998,8 +1009,8 @@ ide_xml_service_get_position_from_cursor_async (IdeXmlService       *self,
                                                 GAsyncReadyCallback  callback,
                                                 gpointer             user_data)
 {
+  g_autoptr(IdeTask) task = NULL;
   PositionState *state;
-  g_autoptr(GTask) task = NULL;
 
   IDE_ENTRY;
 
@@ -1008,21 +1019,23 @@ ide_xml_service_get_position_from_cursor_async (IdeXmlService       *self,
   g_return_if_fail (IDE_IS_BUFFER (buffer) || buffer == NULL);
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-  task = g_task_new (self, cancellable, callback, user_data);
+  task = ide_task_new (self, cancellable, callback, user_data);
+  ide_task_set_source_tag (task, ide_xml_service_get_position_from_cursor_async);
 
   state = g_slice_new0 (PositionState);
-  state->task = g_steal_pointer (&task);
   state->ifile = g_object_ref (ifile);
   state->buffer = g_object_ref (buffer);
   state->line = line;
   state->line_offset = line_offset;
 
+  ide_task_set_task_data (task, state, (GDestroyNotify)position_state_free);
+
   ide_xml_service_get_analysis_async (self,
                                       ifile,
                                       buffer,
                                       cancellable,
                                       ide_xml_service_get_position_from_cursor_cb,
-                                      state);
+                                      g_steal_pointer (&task));
 
   IDE_EXIT;
 }
@@ -1032,13 +1045,10 @@ ide_xml_service_get_position_from_cursor_finish (IdeXmlService  *self,
                                                  GAsyncResult   *result,
                                                  GError        **error)
 {
-  GTask *task = (GTask *)result;
-
   g_return_val_if_fail (IDE_IS_XML_SERVICE (self), NULL);
-  g_return_val_if_fail (G_IS_TASK (result), NULL);
-  g_return_val_if_fail (error != NULL, NULL);
+  g_return_val_if_fail (IDE_IS_TASK (result), NULL);
 
-  return g_task_propagate_pointer (task, error);
+  return ide_task_propagate_pointer (IDE_TASK (result), error);
 }
 
 static void
@@ -1122,22 +1132,11 @@ service_iface_init (IdeServiceInterface *iface)
   iface->stop = ide_xml_service_stop;
 }
 
-static void
-ide_xml_service_class_finalize (IdeXmlServiceClass *klass)
-{
-}
-
 static void
 ide_xml_service_init (IdeXmlService *self)
 {
 }
 
-void
-_ide_xml_service_register_type (GTypeModule *module)
-{
-  ide_xml_service_register_type (module);
-}
-
 /**
  * ide_xml_service_get_cached_root_node:
  *
